/****************************************************************************
** Meta object code from reading C++ file 'qpositioningquickmodule_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioningquick/qpositioningquickmodule_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qpositioningquickmodule_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21QGeoCoordinateForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoCoordinateForeign::qt_create_metaobjectdata<qt_meta_tag_ZN21QGeoCoordinateForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoCoordinateForeign",
        "QML.Foreign",
        "QGeoCoordinate",
        "QML.Element",
        "geoCoordinate",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QGeoCoordinateForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoCoordinateForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QGeoCoordinateForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QGeoCoordinateForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21QGeoCoordinateForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN18QGeoAddressForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoAddressForeign::qt_create_metaobjectdata<qt_meta_tag_ZN18QGeoAddressForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoAddressForeign",
        "QML.Foreign",
        "QGeoAddress",
        "QML.Element",
        "geoAddress",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QGeoAddressForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoAddressForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QGeoAddressForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QGeoAddressForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QGeoAddressForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN20QGeoRectangleForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoRectangleForeign::qt_create_metaobjectdata<qt_meta_tag_ZN20QGeoRectangleForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoRectangleForeign",
        "QML.Foreign",
        "QGeoRectangle",
        "QML.Element",
        "geoRectangle",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QGeoRectangleForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoRectangleForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QGeoRectangleForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QGeoRectangleForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QGeoRectangleForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN17QGeoCircleForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoCircleForeign::qt_create_metaobjectdata<qt_meta_tag_ZN17QGeoCircleForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoCircleForeign",
        "QML.Foreign",
        "QGeoCircle",
        "QML.Element",
        "geoCircle",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QGeoCircleForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoCircleForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QGeoCircleForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QGeoCircleForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QGeoCircleForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN15QGeoPathForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoPathForeign::qt_create_metaobjectdata<qt_meta_tag_ZN15QGeoPathForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoPathForeign",
        "QML.Foreign",
        "QGeoPath",
        "QML.Element",
        "geoPath",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QGeoPathForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoPathForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QGeoPathForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QGeoPathForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QGeoPathForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN18QGeoPolygonForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoPolygonForeign::qt_create_metaobjectdata<qt_meta_tag_ZN18QGeoPolygonForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoPolygonForeign",
        "QML.Foreign",
        "QGeoPolygon",
        "QML.Element",
        "geoPolygon",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QGeoPolygonForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoPolygonForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QGeoPolygonForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QGeoPolygonForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QGeoPolygonForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN19QGeoLocationForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoLocationForeign::qt_create_metaobjectdata<qt_meta_tag_ZN19QGeoLocationForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoLocationForeign",
        "QML.Foreign",
        "QGeoLocation",
        "QML.Element",
        "geoLocation",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QGeoLocationForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoLocationForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QGeoLocationForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QGeoLocationForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QGeoLocationForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN16QGeoShapeForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoShapeForeign::qt_create_metaobjectdata<qt_meta_tag_ZN16QGeoShapeForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoShapeForeign",
        "QML.Foreign",
        "QGeoShape",
        "QML.Element",
        "geoShape",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "construct",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QGeoShapeForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoShapeForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QGeoShapeForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QGeoShapeForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QGeoShapeForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN27QGeoCoordinateObjectForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoCoordinateObjectForeign::qt_create_metaobjectdata<qt_meta_tag_ZN27QGeoCoordinateObjectForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoCoordinateObjectForeign",
        "QML.Foreign",
        "QGeoCoordinateObject",
        "QML.Element",
        "anonymous",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QGeoCoordinateObjectForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoCoordinateObjectForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QGeoCoordinateObjectForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QGeoCoordinateObjectForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QGeoCoordinateObjectForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN23QGeoPositionInfoForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoPositionInfoForeign::qt_create_metaobjectdata<qt_meta_tag_ZN23QGeoPositionInfoForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoPositionInfoForeign",
        "QML.Foreign",
        "QGeoPositionInfo",
        "QML.Element",
        "geoPositionInfo",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QGeoPositionInfoForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoPositionInfoForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QGeoPositionInfoForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QGeoPositionInfoForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23QGeoPositionInfoForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN24QGeoSatelliteInfoForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoSatelliteInfoForeign::qt_create_metaobjectdata<qt_meta_tag_ZN24QGeoSatelliteInfoForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoSatelliteInfoForeign",
        "QML.Foreign",
        "QGeoSatelliteInfo",
        "QML.Element",
        "geoSatelliteInfo",
        "QML.AddedInVersion",
        "1541"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QGeoSatelliteInfoForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoSatelliteInfoForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QGeoSatelliteInfoForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QGeoSatelliteInfoForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QGeoSatelliteInfoForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN24QGeoSatelliteInfoDerivedE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoSatelliteInfoDerived::qt_create_metaobjectdata<qt_meta_tag_ZN24QGeoSatelliteInfoDerivedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoSatelliteInfoDerived"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGeoSatelliteInfoDerived, qt_meta_tag_ZN24QGeoSatelliteInfoDerivedE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoSatelliteInfoDerived::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QGeoSatelliteInfo>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QGeoSatelliteInfoDerivedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QGeoSatelliteInfoDerivedE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QGeoSatelliteInfoDerivedE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN33QGeoSatelliteInfoForeignNamespaceE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoSatelliteInfoForeignNamespace::qt_create_metaobjectdata<qt_meta_tag_ZN33QGeoSatelliteInfoForeignNamespaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoSatelliteInfoForeignNamespace",
        "QML.Foreign",
        "QGeoSatelliteInfoDerived",
        "QML.ForeignIsNamespace",
        "true",
        "QML.Element",
        "GeoSatelliteInfo",
        "QML.AddedInVersion",
        "1541"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN33QGeoSatelliteInfoForeignNamespaceE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN33QGeoSatelliteInfoForeignNamespaceE =
    QGeoSatelliteInfoForeignNamespace::qt_create_metaobjectdata<qt_meta_tag_ZN33QGeoSatelliteInfoForeignNamespaceE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN33QGeoSatelliteInfoForeignNamespaceE =
    qt_staticMetaObjectContent_ZN33QGeoSatelliteInfoForeignNamespaceE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN33QGeoSatelliteInfoForeignNamespaceE =
    qt_staticMetaObjectContent_ZN33QGeoSatelliteInfoForeignNamespaceE.relocatingData;

Q_CONSTINIT const QMetaObject QGeoSatelliteInfoForeignNamespace::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN33QGeoSatelliteInfoForeignNamespaceE.stringdata,
    qt_staticMetaObjectStaticContent_ZN33QGeoSatelliteInfoForeignNamespaceE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN33QGeoSatelliteInfoForeignNamespaceE.metaTypes,
    nullptr
} };

QT_WARNING_POP
