/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeolocation_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioningquick/qdeclarativegeolocation_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeolocation_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23QDeclarativeGeoLocationE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoLocation::qt_create_metaobjectdata<qt_meta_tag_ZN23QDeclarativeGeoLocationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoLocation",
        "QML.Element",
        "Location",
        "QML.AddedInVersion",
        "1280",
        "location",
        "QGeoLocation",
        "address",
        "QDeclarativeGeoAddress*",
        "coordinate",
        "QGeoCoordinate",
        "boundingShape",
        "QGeoShape",
        "extendedAttributes",
        "QVariantMap"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'location'
        QtMocHelpers::PropertyData<QGeoLocation>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'address'
        QtMocHelpers::PropertyData<QDeclarativeGeoAddress*>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable),
        // property 'coordinate'
        QtMocHelpers::PropertyData<QGeoCoordinate>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable),
        // property 'boundingShape'
        QtMocHelpers::PropertyData<QGeoShape>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, -1, 0x602),
        // property 'extendedAttributes'
        QtMocHelpers::PropertyData<QVariantMap>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, -1, 0x50d),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoLocation, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeGeoLocation::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QDeclarativeGeoLocationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QDeclarativeGeoLocationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23QDeclarativeGeoLocationE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoLocation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoLocation *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoAddress* >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoCoordinate >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoLocation >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoShape >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGeoLocation*>(_v) = _t->location(); break;
        case 1: *reinterpret_cast<QDeclarativeGeoAddress**>(_v) = _t->address(); break;
        case 2: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->coordinate(); break;
        case 3: *reinterpret_cast<QGeoShape*>(_v) = _t->boundingShape(); break;
        case 4: *reinterpret_cast<QVariantMap*>(_v) = _t->extendedAttributes(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLocation(*reinterpret_cast<QGeoLocation*>(_v)); break;
        case 1: _t->setAddress(*reinterpret_cast<QDeclarativeGeoAddress**>(_v)); break;
        case 2: _t->setCoordinate(*reinterpret_cast<QGeoCoordinate*>(_v)); break;
        case 3: _t->setBoundingShape(*reinterpret_cast<QGeoShape*>(_v)); break;
        case 4: _t->setExtendedAttributes(*reinterpret_cast<QVariantMap*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 1: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableAddress(); break;
        case 2: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableCoordinate(); break;
        case 3: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableBoundingShape(); break;
        case 4: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableExtendedAttributes(); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoLocation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoLocation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QDeclarativeGeoLocationE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeGeoLocation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
