/****************************************************************************
** Meta object code from reading C++ file 'qgeopolygon.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioning/qgeopolygon.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeopolygon.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QGeoPolygonE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoPolygon::qt_create_metaobjectdata<qt_meta_tag_ZN11QGeoPolygonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoPolygon",
        "addHole",
        "",
        "QVariant",
        "holePath",
        "hole",
        "QVariantList",
        "qsizetype",
        "index",
        "removeHole",
        "holesCount",
        "translate",
        "degreesLatitude",
        "degreesLongitude",
        "translated",
        "length",
        "indexFrom",
        "indexTo",
        "size",
        "addCoordinate",
        "QGeoCoordinate",
        "coordinate",
        "insertCoordinate",
        "replaceCoordinate",
        "coordinateAt",
        "containsCoordinate",
        "removeCoordinate",
        "toString",
        "perimeter",
        "QList<QGeoCoordinate>"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'addHole'
        QtMocHelpers::MethodData<void(const QVariant &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'hole'
        QtMocHelpers::MethodData<const QVariantList(qsizetype) const>(5, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'removeHole'
        QtMocHelpers::MethodData<void(qsizetype)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'holesCount'
        QtMocHelpers::MethodData<qsizetype() const>(10, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Method 'translate'
        QtMocHelpers::MethodData<void(double, double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 12 }, { QMetaType::Double, 13 },
        }}),
        // Method 'translated'
        QtMocHelpers::MethodData<QGeoPolygon(double, double) const>(14, 2, QMC::AccessPublic, 0x80000000 | 0, {{
            { QMetaType::Double, 12 }, { QMetaType::Double, 13 },
        }}),
        // Method 'length'
        QtMocHelpers::MethodData<double(qsizetype, qsizetype) const>(15, 2, QMC::AccessPublic, QMetaType::Double, {{
            { 0x80000000 | 7, 16 }, { 0x80000000 | 7, 17 },
        }}),
        // Method 'length'
        QtMocHelpers::MethodData<double(qsizetype) const>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Double, {{
            { 0x80000000 | 7, 16 },
        }}),
        // Method 'length'
        QtMocHelpers::MethodData<double() const>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Double),
        // Method 'size'
        QtMocHelpers::MethodData<qsizetype() const>(18, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Method 'addCoordinate'
        QtMocHelpers::MethodData<void(const QGeoCoordinate &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Method 'insertCoordinate'
        QtMocHelpers::MethodData<void(qsizetype, const QGeoCoordinate &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 20, 21 },
        }}),
        // Method 'replaceCoordinate'
        QtMocHelpers::MethodData<void(qsizetype, const QGeoCoordinate &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 20, 21 },
        }}),
        // Method 'coordinateAt'
        QtMocHelpers::MethodData<QGeoCoordinate(qsizetype) const>(24, 2, QMC::AccessPublic, 0x80000000 | 20, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'containsCoordinate'
        QtMocHelpers::MethodData<bool(const QGeoCoordinate &) const>(25, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Method 'removeCoordinate'
        QtMocHelpers::MethodData<void(const QGeoCoordinate &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Method 'removeCoordinate'
        QtMocHelpers::MethodData<void(qsizetype)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'toString'
        QtMocHelpers::MethodData<QString() const>(27, 2, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'perimeter'
        QtMocHelpers::PropertyData<QList<QGeoCoordinate>>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, -1, 0x50c),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGeoPolygon, qt_meta_tag_ZN11QGeoPolygonE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoPolygon::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QGeoShape>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGeoPolygonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGeoPolygonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QGeoPolygonE_t>.metaTypes,
    nullptr
} };

void QGeoPolygon::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGeoPolygon *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addHole((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 1: { QVariantList _r = _t->hole((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->removeHole((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 3: { qsizetype _r = _t->holesCount();
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->translate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 5: { QGeoPolygon _r = _t->translated((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoPolygon*>(_a[0]) = std::move(_r); }  break;
        case 6: { double _r = _t->length((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 7: { double _r = _t->length((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 8: { double _r = _t->length();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 9: { qsizetype _r = _t->size();
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->addCoordinate((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 11: _t->insertCoordinate((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[2]))); break;
        case 12: _t->replaceCoordinate((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[2]))); break;
        case 13: { QGeoCoordinate _r = _t->coordinateAt((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->containsCoordinate((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->removeCoordinate((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 16: _t->removeCoordinate((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 17: { QString _r = _t->toString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QGeoCoordinate> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QGeoCoordinate>*>(_v) = _t->perimeter(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPerimeter(*reinterpret_cast<QList<QGeoCoordinate>*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
