/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.GeoClue2.Client.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef CLIENT_INTERFACE_H
#define CLIENT_INTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "geocluetypes.h"

namespace QtPositioningPrivate {

/*
 * Proxy class for interface org.freedesktop.GeoClue2.Client
 */
class OrgFreedesktopGeoClue2ClientInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.GeoClue2.Client"; }

public:
    OrgFreedesktopGeoClue2ClientInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopGeoClue2ClientInterface();

    Q_PROPERTY(bool Active READ active)
    inline bool active() const
    { return qvariant_cast< bool >(property("Active")); }

    Q_PROPERTY(QString DesktopId READ desktopId WRITE setDesktopId)
    inline QString desktopId() const
    { return qvariant_cast< QString >(property("DesktopId")); }
    inline void setDesktopId(const QString &value)
    { setProperty("DesktopId", QVariant::fromValue(value)); }

    Q_PROPERTY(uint DistanceThreshold READ distanceThreshold WRITE setDistanceThreshold)
    inline uint distanceThreshold() const
    { return qvariant_cast< uint >(property("DistanceThreshold")); }
    inline void setDistanceThreshold(uint value)
    { setProperty("DistanceThreshold", QVariant::fromValue(value)); }

    Q_PROPERTY(QDBusObjectPath Location READ location)
    inline QDBusObjectPath location() const
    { return qvariant_cast< QDBusObjectPath >(property("Location")); }

    Q_PROPERTY(uint RequestedAccuracyLevel READ requestedAccuracyLevel WRITE setRequestedAccuracyLevel)
    inline uint requestedAccuracyLevel() const
    { return qvariant_cast< uint >(property("RequestedAccuracyLevel")); }
    inline void setRequestedAccuracyLevel(uint value)
    { setProperty("RequestedAccuracyLevel", QVariant::fromValue(value)); }

    Q_PROPERTY(uint TimeThreshold READ timeThreshold WRITE setTimeThreshold)
    inline uint timeThreshold() const
    { return qvariant_cast< uint >(property("TimeThreshold")); }
    inline void setTimeThreshold(uint value)
    { setProperty("TimeThreshold", QVariant::fromValue(value)); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> Start()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Start"), argumentList);
    }

    inline QDBusPendingReply<> Stop()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Stop"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void LocationUpdated(const QDBusObjectPath &oldLocation, const QDBusObjectPath &newLocation);
};

} // end of namespace QtPositioningPrivate

#endif
