/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/locationsingleton_p.h>)
#  include <private/locationsingleton_p.h>
#endif
#if __has_include(<private/qdeclarativegeoaddress_p.h>)
#  include <private/qdeclarativegeoaddress_p.h>
#endif
#if __has_include(<private/qdeclarativegeolocation_p.h>)
#  include <private/qdeclarativegeolocation_p.h>
#endif
#if __has_include(<private/qdeclarativepluginparameter_p.h>)
#  include <private/qdeclarativepluginparameter_p.h>
#endif
#if __has_include(<private/qdeclarativeposition_p.h>)
#  include <private/qdeclarativeposition_p.h>
#endif
#if __has_include(<private/qdeclarativepositionsource_p.h>)
#  include <private/qdeclarativepositionsource_p.h>
#endif
#if __has_include(<private/qdeclarativesatellitesource_p.h>)
#  include <private/qdeclarativesatellitesource_p.h>
#endif
#if __has_include(<private/qpositioningquickmodule_p.h>)
#  include <private/qpositioningquickmodule_p.h>
#endif
#if __has_include(<private/qquickgeocoordinateanimation_p.h>)
#  include <private/qquickgeocoordinateanimation_p.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtPositioning()
{
    qmlRegisterModule("QtPositioning", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<LocationSingleton>("QtPositioning", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoAddress>("QtPositioning", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoLocation>("QtPositioning", 6);
    qmlRegisterTypesAndRevisions<QDeclarativePluginParameter>("QtPositioning", 6);
    qmlRegisterTypesAndRevisions<QDeclarativePosition>("QtPositioning", 6);
    qmlRegisterTypesAndRevisions<QDeclarativePositionSource>("QtPositioning", 6);
    qmlRegisterEnum<QDeclarativePositionSource::PositioningMethod>("QDeclarativePositionSource::PositioningMethod");
    qmlRegisterEnum<QDeclarativePositionSource::PositioningMethods>("QDeclarativePositionSource::PositioningMethods");
    qmlRegisterEnum<QDeclarativePositionSource::PositioningMethod>("QDeclarativePositionSource::PositioningMethod");
    qmlRegisterEnum<QDeclarativePositionSource::SourceError>("QDeclarativePositionSource::SourceError");
    qmlRegisterTypesAndRevisions<QDeclarativeSatelliteSource>("QtPositioning", 6);
    qmlRegisterEnum<QDeclarativeSatelliteSource::SourceError>("QDeclarativeSatelliteSource::SourceError");
    qmlRegisterTypesAndRevisions<QGeoAddressForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoAddressForeign *>().id();
    qmlRegisterTypesAndRevisions<QGeoCircleForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoCircleForeign *>().id();
    qmlRegisterTypesAndRevisions<QGeoCoordinateForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoCoordinateForeign *>().id();
    qmlRegisterEnum<QGeoCoordinate::CoordinateFormat>("QGeoCoordinate::CoordinateFormat");
    qmlRegisterTypesAndRevisions<QGeoCoordinateObjectForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoCoordinateObjectForeign *>().id();
    qmlRegisterTypesAndRevisions<QGeoLocationForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoLocationForeign *>().id();
    qmlRegisterTypesAndRevisions<QGeoPathForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoPathForeign *>().id();
    qmlRegisterTypesAndRevisions<QGeoPolygonForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoPolygonForeign *>().id();
    qmlRegisterTypesAndRevisions<QGeoPositionInfoForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoPositionInfoForeign *>().id();
    qmlRegisterTypesAndRevisions<QGeoRectangleForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoRectangleForeign *>().id();
    qmlRegisterTypesAndRevisions<QGeoSatelliteInfoForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoSatelliteInfoForeign *>().id();
    qmlRegisterEnum<QGeoSatelliteInfo::Attribute>("QGeoSatelliteInfo::Attribute");
    qmlRegisterEnum<QGeoSatelliteInfo::SatelliteSystem>("QGeoSatelliteInfo::SatelliteSystem");
    QMetaType::fromType<QGeoSatelliteInfoDerived>().id();
    qmlRegisterNamespaceAndRevisions(&QGeoSatelliteInfoDerived::staticMetaObject, "QtPositioning", 6, nullptr, &QGeoSatelliteInfoForeignNamespace::staticMetaObject, nullptr);
    qmlRegisterTypesAndRevisions<QGeoShapeForeign>("QtPositioning", 6);
    QMetaType::fromType<QGeoShapeForeign *>().id();
    qmlRegisterEnum<QGeoShape::ShapeType>("QGeoShape::ShapeType");
    qmlRegisterTypesAndRevisions<QQuickGeoCoordinateAnimation>("QtPositioning", 6);
    qmlRegisterAnonymousTypesAndRevisions<QQuickAbstractAnimation>("QtPositioning", 6);
    qmlRegisterEnum<QQuickGeoCoordinateAnimation::Direction>("QQuickGeoCoordinateAnimation::Direction");
    QT_WARNING_POP
    qmlRegisterModule("QtPositioning", 6, 12);
}

static const QQmlModuleRegistration qtPositioningRegistration("QtPositioning", qml_register_types_QtPositioning);
