/****************************************************************************
** Meta object code from reading C++ file 'client_interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../client_interface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'client_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20QtPositioningPrivate37OrgFreedesktopGeoClue2ClientInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto QtPositioningPrivate::OrgFreedesktopGeoClue2ClientInterface::qt_create_metaobjectdata<qt_meta_tag_ZN20QtPositioningPrivate37OrgFreedesktopGeoClue2ClientInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtPositioningPrivate::OrgFreedesktopGeoClue2ClientInterface",
        "LocationUpdated",
        "",
        "QDBusObjectPath",
        "oldLocation",
        "newLocation",
        "Start",
        "QDBusPendingReply<>",
        "Stop",
        "Active",
        "DesktopId",
        "DistanceThreshold",
        "Location",
        "RequestedAccuracyLevel",
        "TimeThreshold"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'LocationUpdated'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QDBusObjectPath &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Slot 'Start'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'Stop'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(8, 2, QMC::AccessPublic, 0x80000000 | 7),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Active'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'DesktopId'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'DistanceThreshold'
        QtMocHelpers::PropertyData<uint>(11, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'Location'
        QtMocHelpers::PropertyData<QDBusObjectPath>(12, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'RequestedAccuracyLevel'
        QtMocHelpers::PropertyData<uint>(13, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'TimeThreshold'
        QtMocHelpers::PropertyData<uint>(14, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopGeoClue2ClientInterface, qt_meta_tag_ZN20QtPositioningPrivate37OrgFreedesktopGeoClue2ClientInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtPositioningPrivate::OrgFreedesktopGeoClue2ClientInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QtPositioningPrivate37OrgFreedesktopGeoClue2ClientInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QtPositioningPrivate37OrgFreedesktopGeoClue2ClientInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QtPositioningPrivate37OrgFreedesktopGeoClue2ClientInterfaceE_t>.metaTypes,
    nullptr
} };

void QtPositioningPrivate::OrgFreedesktopGeoClue2ClientInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopGeoClue2ClientInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->LocationUpdated((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 1: { QDBusPendingReply<> _r = _t->Start();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->Stop();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopGeoClue2ClientInterface::*)(const QDBusObjectPath & , const QDBusObjectPath & )>(_a, &OrgFreedesktopGeoClue2ClientInterface::LocationUpdated, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->active(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->desktopId(); break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->distanceThreshold(); break;
        case 3: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->location(); break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->requestedAccuracyLevel(); break;
        case 5: *reinterpret_cast<uint*>(_v) = _t->timeThreshold(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setDesktopId(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setDistanceThreshold(*reinterpret_cast<uint*>(_v)); break;
        case 4: _t->setRequestedAccuracyLevel(*reinterpret_cast<uint*>(_v)); break;
        case 5: _t->setTimeThreshold(*reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QtPositioningPrivate::OrgFreedesktopGeoClue2ClientInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtPositioningPrivate::OrgFreedesktopGeoClue2ClientInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QtPositioningPrivate37OrgFreedesktopGeoClue2ClientInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int QtPositioningPrivate::OrgFreedesktopGeoClue2ClientInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QtPositioningPrivate::OrgFreedesktopGeoClue2ClientInterface::LocationUpdated(const QDBusObjectPath & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
