/****************************************************************************
** Meta object code from reading C++ file 'qgeosatelliteinfo.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioning/qgeosatelliteinfo.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeosatelliteinfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QGeoSatelliteInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoSatelliteInfo::qt_create_metaobjectdata<qt_meta_tag_ZN17QGeoSatelliteInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoSatelliteInfo",
        "attribute",
        "",
        "Attribute",
        "hasAttribute",
        "satelliteSystem",
        "SatelliteSystem",
        "satelliteIdentifier",
        "signalStrength",
        "Elevation",
        "Azimuth",
        "Undefined",
        "GPS",
        "GLONASS",
        "GALILEO",
        "BEIDOU",
        "QZSS",
        "IRNSS",
        "SBAS",
        "Multiple",
        "CustomType"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'attribute'
        QtMocHelpers::MethodData<qreal(enum Attribute) const>(1, 2, QMC::AccessPublic, QMetaType::QReal, {{
            { 0x80000000 | 3, 1 },
        }}),
        // Method 'hasAttribute'
        QtMocHelpers::MethodData<bool(enum Attribute) const>(4, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 3, 1 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'satelliteSystem'
        QtMocHelpers::PropertyData<enum SatelliteSystem>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final),
        // property 'satelliteIdentifier'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Final),
        // property 'signalStrength'
        QtMocHelpers::PropertyData<qreal>(8, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Attribute'
        QtMocHelpers::EnumData<enum Attribute>(3, 3, QMC::EnumFlags{}).add({
            {    9, Attribute::Elevation },
            {   10, Attribute::Azimuth },
        }),
        // enum 'SatelliteSystem'
        QtMocHelpers::EnumData<enum SatelliteSystem>(6, 6, QMC::EnumFlags{}).add({
            {   11, SatelliteSystem::Undefined },
            {   12, SatelliteSystem::GPS },
            {   13, SatelliteSystem::GLONASS },
            {   14, SatelliteSystem::GALILEO },
            {   15, SatelliteSystem::BEIDOU },
            {   16, SatelliteSystem::QZSS },
            {   17, SatelliteSystem::IRNSS },
            {   18, SatelliteSystem::SBAS },
            {   19, SatelliteSystem::Multiple },
            {   20, SatelliteSystem::CustomType },
        }),
    };
    return QtMocHelpers::metaObjectData<QGeoSatelliteInfo, qt_meta_tag_ZN17QGeoSatelliteInfoE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoSatelliteInfo::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QGeoSatelliteInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QGeoSatelliteInfoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QGeoSatelliteInfoE_t>.metaTypes,
    nullptr
} };

void QGeoSatelliteInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGeoSatelliteInfo *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { qreal _r = _t->attribute((*reinterpret_cast<std::add_pointer_t<enum Attribute>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->hasAttribute((*reinterpret_cast<std::add_pointer_t<enum Attribute>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum SatelliteSystem*>(_v) = _t->satelliteSystem(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->satelliteIdentifier(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->signalStrength(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
