/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativesatellitesource_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioningquick/qdeclarativesatellitesource_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativesatellitesource_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27QDeclarativeSatelliteSourceE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeSatelliteSource::qt_create_metaobjectdata<qt_meta_tag_ZN27QDeclarativeSatelliteSourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeSatelliteSource",
        "QML.Element",
        "SatelliteSource",
        "QML.AddedInVersion",
        "1541",
        "DefaultProperty",
        "parameters",
        "activeChanged",
        "",
        "validityChanged",
        "updateIntervalChanged",
        "sourceErrorChanged",
        "nameChanged",
        "satellitesInUseChanged",
        "satellitesInViewChanged",
        "update",
        "timeout",
        "start",
        "stop",
        "sourceErrorReceived",
        "QGeoSatelliteInfoSource::Error",
        "error",
        "onParameterInitialized",
        "satellitesInViewUpdateReceived",
        "QList<QGeoSatelliteInfo>",
        "satellites",
        "satellitesInUseUpdateReceived",
        "setBackendProperty",
        "name",
        "QVariant",
        "value",
        "backendProperty",
        "active",
        "valid",
        "updateInterval",
        "sourceError",
        "SourceError",
        "QQmlListProperty<QDeclarativePluginParameter>",
        "satellitesInUse",
        "satellitesInView",
        "AccessError",
        "ClosedError",
        "NoError",
        "UnknownSourceError",
        "UpdateTimeoutError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activeChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'validityChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateIntervalChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceErrorChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'satellitesInUseChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'satellitesInViewChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void(int)>(15, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(15, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sourceErrorReceived'
        QtMocHelpers::SlotData<void(const QGeoSatelliteInfoSource::Error)>(19, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'onParameterInitialized'
        QtMocHelpers::SlotData<void()>(22, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'satellitesInViewUpdateReceived'
        QtMocHelpers::SlotData<void(const QList<QGeoSatelliteInfo> &)>(23, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'satellitesInUseUpdateReceived'
        QtMocHelpers::SlotData<void(const QList<QGeoSatelliteInfo> &)>(26, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Method 'setBackendProperty'
        QtMocHelpers::MethodData<bool(const QString &, const QVariant &)>(27, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'backendProperty'
        QtMocHelpers::MethodData<QVariant(const QString &) const>(31, 8, QMC::AccessPublic, 0x80000000 | 29, {{
            { QMetaType::QString, 28 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'active'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'valid'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'updateInterval'
        QtMocHelpers::PropertyData<int>(34, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'sourceError'
        QtMocHelpers::PropertyData<enum SourceError>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'parameters'
        QtMocHelpers::PropertyData<QQmlListProperty<QDeclarativePluginParameter>>(6, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'satellitesInUse'
        QtMocHelpers::PropertyData<QList<QGeoSatelliteInfo>>(38, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'satellitesInView'
        QtMocHelpers::PropertyData<QList<QGeoSatelliteInfo>>(39, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 6),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SourceError'
        QtMocHelpers::EnumData<enum SourceError>(36, 36, QMC::EnumFlags{}).add({
            {   40, SourceError::AccessError },
            {   41, SourceError::ClosedError },
            {   42, SourceError::NoError },
            {   43, SourceError::UnknownSourceError },
            {   44, SourceError::UpdateTimeoutError },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeSatelliteSource, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeSatelliteSource::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativeSatelliteSourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativeSatelliteSourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QDeclarativeSatelliteSourceE_t>.metaTypes,
    nullptr
} };

void QDeclarativeSatelliteSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeSatelliteSource *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activeChanged(); break;
        case 1: _t->validityChanged(); break;
        case 2: _t->updateIntervalChanged(); break;
        case 3: _t->sourceErrorChanged(); break;
        case 4: _t->nameChanged(); break;
        case 5: _t->satellitesInUseChanged(); break;
        case 6: _t->satellitesInViewChanged(); break;
        case 7: _t->update((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->update(); break;
        case 9: _t->start(); break;
        case 10: _t->stop(); break;
        case 11: _t->sourceErrorReceived((*reinterpret_cast<std::add_pointer_t<QGeoSatelliteInfoSource::Error>>(_a[1]))); break;
        case 12: _t->onParameterInitialized(); break;
        case 13: _t->satellitesInViewUpdateReceived((*reinterpret_cast<std::add_pointer_t<QList<QGeoSatelliteInfo>>>(_a[1]))); break;
        case 14: _t->satellitesInUseUpdateReceived((*reinterpret_cast<std::add_pointer_t<QList<QGeoSatelliteInfo>>>(_a[1]))); break;
        case 15: { bool _r = _t->setBackendProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { QVariant _r = _t->backendProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QGeoSatelliteInfo> >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QGeoSatelliteInfo> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSatelliteSource::*)()>(_a, &QDeclarativeSatelliteSource::activeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSatelliteSource::*)()>(_a, &QDeclarativeSatelliteSource::validityChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSatelliteSource::*)()>(_a, &QDeclarativeSatelliteSource::updateIntervalChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSatelliteSource::*)()>(_a, &QDeclarativeSatelliteSource::sourceErrorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSatelliteSource::*)()>(_a, &QDeclarativeSatelliteSource::nameChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSatelliteSource::*)()>(_a, &QDeclarativeSatelliteSource::satellitesInUseChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSatelliteSource::*)()>(_a, &QDeclarativeSatelliteSource::satellitesInViewChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QGeoSatelliteInfo> >(); break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QDeclarativePluginParameter> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isActive(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->updateInterval(); break;
        case 3: *reinterpret_cast<enum SourceError*>(_v) = _t->sourceError(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 5: *reinterpret_cast<QQmlListProperty<QDeclarativePluginParameter>*>(_v) = _t->parameters(); break;
        case 6: *reinterpret_cast<QList<QGeoSatelliteInfo>*>(_v) = _t->satellitesInUse(); break;
        case 7: *reinterpret_cast<QList<QGeoSatelliteInfo>*>(_v) = _t->satellitesInView(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setActive(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setUpdateInterval(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeSatelliteSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeSatelliteSource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativeSatelliteSourceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeSatelliteSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeSatelliteSource::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeSatelliteSource::validityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeSatelliteSource::updateIntervalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeSatelliteSource::sourceErrorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeSatelliteSource::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeSatelliteSource::satellitesInUseChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeSatelliteSource::satellitesInViewChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
