/****************************************************************************
** Meta object code from reading C++ file 'locationsingleton_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioningquick/locationsingleton_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'locationsingleton_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17LocationSingletonE_t {};
} // unnamed namespace

template <> constexpr inline auto LocationSingleton::qt_create_metaobjectdata<qt_meta_tag_ZN17LocationSingletonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LocationSingleton",
        "QML.Element",
        "QtPositioning",
        "QML.Singleton",
        "true",
        "QML.AddedInVersion",
        "1280",
        "coordinate",
        "QGeoCoordinate",
        "",
        "latitude",
        "longitude",
        "altitude",
        "shape",
        "QGeoShape",
        "rectangle",
        "QGeoRectangle",
        "center",
        "width",
        "height",
        "topLeft",
        "bottomRight",
        "QVariantList",
        "coordinates",
        "circle",
        "QGeoCircle",
        "radius",
        "path",
        "QGeoPath",
        "QJSValue",
        "value",
        "polygon",
        "QGeoPolygon",
        "perimeter",
        "holes",
        "shapeToCircle",
        "shapeToRectangle",
        "shapeToPath",
        "shapeToPolygon",
        "mercatorToCoord",
        "QPointF",
        "mercator",
        "coordToMercator",
        "coord"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'coordinate'
        QtMocHelpers::MethodData<QGeoCoordinate() const>(7, 9, QMC::AccessPublic, 0x80000000 | 8),
        // Method 'coordinate'
        QtMocHelpers::MethodData<QGeoCoordinate(double, double, double) const>(7, 9, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::Double, 10 }, { QMetaType::Double, 11 }, { QMetaType::Double, 12 },
        }}),
        // Method 'coordinate'
        QtMocHelpers::MethodData<QGeoCoordinate(double, double) const>(7, 9, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 8, {{
            { QMetaType::Double, 10 }, { QMetaType::Double, 11 },
        }}),
        // Method 'shape'
        QtMocHelpers::MethodData<QGeoShape() const>(13, 9, QMC::AccessPublic, 0x80000000 | 14),
        // Method 'rectangle'
        QtMocHelpers::MethodData<QGeoRectangle() const>(15, 9, QMC::AccessPublic, 0x80000000 | 16),
        // Method 'rectangle'
        QtMocHelpers::MethodData<QGeoRectangle(const QGeoCoordinate &, double, double) const>(15, 9, QMC::AccessPublic, 0x80000000 | 16, {{
            { 0x80000000 | 8, 17 }, { QMetaType::Double, 18 }, { QMetaType::Double, 19 },
        }}),
        // Method 'rectangle'
        QtMocHelpers::MethodData<QGeoRectangle(const QGeoCoordinate &, const QGeoCoordinate &) const>(15, 9, QMC::AccessPublic, 0x80000000 | 16, {{
            { 0x80000000 | 8, 20 }, { 0x80000000 | 8, 21 },
        }}),
        // Method 'rectangle'
        QtMocHelpers::MethodData<QGeoRectangle(const QVariantList &) const>(15, 9, QMC::AccessPublic, 0x80000000 | 16, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Method 'circle'
        QtMocHelpers::MethodData<QGeoCircle() const>(24, 9, QMC::AccessPublic, 0x80000000 | 25),
        // Method 'circle'
        QtMocHelpers::MethodData<QGeoCircle(const QGeoCoordinate &, qreal) const>(24, 9, QMC::AccessPublic, 0x80000000 | 25, {{
            { 0x80000000 | 8, 17 }, { QMetaType::QReal, 26 },
        }}),
        // Method 'circle'
        QtMocHelpers::MethodData<QGeoCircle(const QGeoCoordinate &) const>(24, 9, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 25, {{
            { 0x80000000 | 8, 17 },
        }}),
        // Method 'path'
        QtMocHelpers::MethodData<QGeoPath() const>(27, 9, QMC::AccessPublic, 0x80000000 | 28),
        // Method 'path'
        QtMocHelpers::MethodData<QGeoPath(const QJSValue &, qreal) const>(27, 9, QMC::AccessPublic, 0x80000000 | 28, {{
            { 0x80000000 | 29, 30 }, { QMetaType::QReal, 18 },
        }}),
        // Method 'path'
        QtMocHelpers::MethodData<QGeoPath(const QJSValue &) const>(27, 9, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 28, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Method 'polygon'
        QtMocHelpers::MethodData<QGeoPolygon() const>(31, 9, QMC::AccessPublic, 0x80000000 | 32),
        // Method 'polygon'
        QtMocHelpers::MethodData<QGeoPolygon(const QVariantList &) const>(31, 9, QMC::AccessPublic, 0x80000000 | 32, {{
            { 0x80000000 | 22, 30 },
        }}),
        // Method 'polygon'
        QtMocHelpers::MethodData<QGeoPolygon(const QVariantList &, const QVariantList &) const>(31, 9, QMC::AccessPublic, 0x80000000 | 32, {{
            { 0x80000000 | 22, 33 }, { 0x80000000 | 22, 34 },
        }}),
        // Method 'shapeToCircle'
        QtMocHelpers::MethodData<QGeoCircle(const QGeoShape &) const>(35, 9, QMC::AccessPublic, 0x80000000 | 25, {{
            { 0x80000000 | 14, 13 },
        }}),
        // Method 'shapeToRectangle'
        QtMocHelpers::MethodData<QGeoRectangle(const QGeoShape &) const>(36, 9, QMC::AccessPublic, 0x80000000 | 16, {{
            { 0x80000000 | 14, 13 },
        }}),
        // Method 'shapeToPath'
        QtMocHelpers::MethodData<QGeoPath(const QGeoShape &) const>(37, 9, QMC::AccessPublic, 0x80000000 | 28, {{
            { 0x80000000 | 14, 13 },
        }}),
        // Method 'shapeToPolygon'
        QtMocHelpers::MethodData<QGeoPolygon(const QGeoShape &) const>(38, 9, QMC::AccessPublic, 0x80000000 | 32, {{
            { 0x80000000 | 14, 13 },
        }}),
        // Method 'mercatorToCoord'
        QtMocHelpers::RevisionedMethodData<QGeoCoordinate(const QPointF &) const>(39, 9, QMC::AccessPublic, 0x50c, 0x80000000 | 8, {{
            { 0x80000000 | 40, 41 },
        }}),
        // Method 'coordToMercator'
        QtMocHelpers::RevisionedMethodData<QPointF(const QGeoCoordinate &) const>(42, 9, QMC::AccessPublic, 0x50c, 0x80000000 | 40, {{
            { 0x80000000 | 8, 43 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<LocationSingleton, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject LocationSingleton::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17LocationSingletonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17LocationSingletonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17LocationSingletonE_t>.metaTypes,
    nullptr
} };

void LocationSingleton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LocationSingleton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QGeoCoordinate _r = _t->coordinate();
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 1: { QGeoCoordinate _r = _t->coordinate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 2: { QGeoCoordinate _r = _t->coordinate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 3: { QGeoShape _r = _t->shape();
            if (_a[0]) *reinterpret_cast<QGeoShape*>(_a[0]) = std::move(_r); }  break;
        case 4: { QGeoRectangle _r = _t->rectangle();
            if (_a[0]) *reinterpret_cast<QGeoRectangle*>(_a[0]) = std::move(_r); }  break;
        case 5: { QGeoRectangle _r = _t->rectangle((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QGeoRectangle*>(_a[0]) = std::move(_r); }  break;
        case 6: { QGeoRectangle _r = _t->rectangle((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoRectangle*>(_a[0]) = std::move(_r); }  break;
        case 7: { QGeoRectangle _r = _t->rectangle((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoRectangle*>(_a[0]) = std::move(_r); }  break;
        case 8: { QGeoCircle _r = _t->circle();
            if (_a[0]) *reinterpret_cast<QGeoCircle*>(_a[0]) = std::move(_r); }  break;
        case 9: { QGeoCircle _r = _t->circle((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoCircle*>(_a[0]) = std::move(_r); }  break;
        case 10: { QGeoCircle _r = _t->circle((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoCircle*>(_a[0]) = std::move(_r); }  break;
        case 11: { QGeoPath _r = _t->path();
            if (_a[0]) *reinterpret_cast<QGeoPath*>(_a[0]) = std::move(_r); }  break;
        case 12: { QGeoPath _r = _t->path((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoPath*>(_a[0]) = std::move(_r); }  break;
        case 13: { QGeoPath _r = _t->path((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoPath*>(_a[0]) = std::move(_r); }  break;
        case 14: { QGeoPolygon _r = _t->polygon();
            if (_a[0]) *reinterpret_cast<QGeoPolygon*>(_a[0]) = std::move(_r); }  break;
        case 15: { QGeoPolygon _r = _t->polygon((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoPolygon*>(_a[0]) = std::move(_r); }  break;
        case 16: { QGeoPolygon _r = _t->polygon((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoPolygon*>(_a[0]) = std::move(_r); }  break;
        case 17: { QGeoCircle _r = _t->shapeToCircle((*reinterpret_cast<std::add_pointer_t<QGeoShape>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoCircle*>(_a[0]) = std::move(_r); }  break;
        case 18: { QGeoRectangle _r = _t->shapeToRectangle((*reinterpret_cast<std::add_pointer_t<QGeoShape>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoRectangle*>(_a[0]) = std::move(_r); }  break;
        case 19: { QGeoPath _r = _t->shapeToPath((*reinterpret_cast<std::add_pointer_t<QGeoShape>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoPath*>(_a[0]) = std::move(_r); }  break;
        case 20: { QGeoPolygon _r = _t->shapeToPolygon((*reinterpret_cast<std::add_pointer_t<QGeoShape>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoPolygon*>(_a[0]) = std::move(_r); }  break;
        case 21: { QGeoCoordinate _r = _t->mercatorToCoord((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 22: { QPointF _r = _t->coordToMercator((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoShape >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoShape >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoShape >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoShape >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        }
    }
}

const QMetaObject *LocationSingleton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LocationSingleton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17LocationSingletonE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LocationSingleton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}
QT_WARNING_POP
