/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.GeoClue2.Location.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef LOCATION_INTERFACE_H
#define LOCATION_INTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "geocluetypes.h"

namespace QtPositioningPrivate {

/*
 * Proxy class for interface org.freedesktop.GeoClue2.Location
 */
class OrgFreedesktopGeoClue2LocationInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.GeoClue2.Location"; }

public:
    OrgFreedesktopGeoClue2LocationInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopGeoClue2LocationInterface();

    Q_PROPERTY(double Accuracy READ accuracy)
    inline double accuracy() const
    { return qvariant_cast< double >(property("Accuracy")); }

    Q_PROPERTY(double Altitude READ altitude)
    inline double altitude() const
    { return qvariant_cast< double >(property("Altitude")); }

    Q_PROPERTY(QString Description READ description)
    inline QString description() const
    { return qvariant_cast< QString >(property("Description")); }

    Q_PROPERTY(double Heading READ heading)
    inline double heading() const
    { return qvariant_cast< double >(property("Heading")); }

    Q_PROPERTY(double Latitude READ latitude)
    inline double latitude() const
    { return qvariant_cast< double >(property("Latitude")); }

    Q_PROPERTY(double Longitude READ longitude)
    inline double longitude() const
    { return qvariant_cast< double >(property("Longitude")); }

    Q_PROPERTY(double Speed READ speed)
    inline double speed() const
    { return qvariant_cast< double >(property("Speed")); }

    Q_PROPERTY(Timestamp Timestamp READ timestamp)
    inline Timestamp timestamp() const
    { return qvariant_cast< Timestamp >(property("Timestamp")); }

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
};

} // end of namespace QtPositioningPrivate

#endif
