/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeoaddress_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioningquick/qdeclarativegeoaddress_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeoaddress_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22QDeclarativeGeoAddressE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoAddress::qt_create_metaobjectdata<qt_meta_tag_ZN22QDeclarativeGeoAddressE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoAddress",
        "QML.Element",
        "Address",
        "QML.AddedInVersion",
        "1280",
        "textChanged",
        "",
        "countryChanged",
        "countryCodeChanged",
        "stateChanged",
        "countyChanged",
        "cityChanged",
        "districtChanged",
        "streetChanged",
        "streetNumberChanged",
        "postalCodeChanged",
        "isTextGeneratedChanged",
        "address",
        "QGeoAddress",
        "text",
        "country",
        "countryCode",
        "state",
        "county",
        "city",
        "district",
        "street",
        "streetNumber",
        "postalCode",
        "isTextGenerated"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'textChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'countryChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'countryCodeChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'countyChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cityChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'districtChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'streetChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'streetNumberChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'postalCodeChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isTextGeneratedChanged'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'address'
        QtMocHelpers::PropertyData<QGeoAddress>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'text'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'country'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'countryCode'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'state'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'county'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'city'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'district'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'street'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'streetNumber'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8, 0x602),
        // property 'postalCode'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'isTextGenerated'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoAddress, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeGeoAddress::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QDeclarativeGeoAddressE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QDeclarativeGeoAddressE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22QDeclarativeGeoAddressE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoAddress::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoAddress *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->textChanged(); break;
        case 1: _t->countryChanged(); break;
        case 2: _t->countryCodeChanged(); break;
        case 3: _t->stateChanged(); break;
        case 4: _t->countyChanged(); break;
        case 5: _t->cityChanged(); break;
        case 6: _t->districtChanged(); break;
        case 7: _t->streetChanged(); break;
        case 8: _t->streetNumberChanged(); break;
        case 9: _t->postalCodeChanged(); break;
        case 10: _t->isTextGeneratedChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::textChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::countryChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::countryCodeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::stateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::countyChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::cityChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::districtChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::streetChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::streetNumberChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::postalCodeChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoAddress::*)()>(_a, &QDeclarativeGeoAddress::isTextGeneratedChanged, 10))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoAddress >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGeoAddress*>(_v) = _t->address(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->text(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->country(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->countryCode(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->state(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->county(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->city(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->district(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->street(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->streetNumber(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->postalCode(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isTextGenerated(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAddress(*reinterpret_cast<QGeoAddress*>(_v)); break;
        case 1: _t->setText(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setCountry(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setCountryCode(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setState(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setCounty(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setCity(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setDistrict(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setStreet(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setStreetNumber(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setPostalCode(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoAddress::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoAddress::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QDeclarativeGeoAddressE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeGeoAddress::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoAddress::textChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeGeoAddress::countryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeGeoAddress::countryCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeGeoAddress::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeGeoAddress::countyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeGeoAddress::cityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeGeoAddress::districtChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDeclarativeGeoAddress::streetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QDeclarativeGeoAddress::streetNumberChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QDeclarativeGeoAddress::postalCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QDeclarativeGeoAddress::isTextGeneratedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
