/****************************************************************************
** Meta object code from reading C++ file 'qgeopath.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioning/qgeopath.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeopath.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8QGeoPathE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoPath::qt_create_metaobjectdata<qt_meta_tag_ZN8QGeoPathE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoPath",
        "translate",
        "",
        "degreesLatitude",
        "degreesLongitude",
        "translated",
        "length",
        "qsizetype",
        "indexFrom",
        "indexTo",
        "size",
        "addCoordinate",
        "QGeoCoordinate",
        "coordinate",
        "insertCoordinate",
        "index",
        "replaceCoordinate",
        "coordinateAt",
        "containsCoordinate",
        "removeCoordinate",
        "toString",
        "path",
        "QVariantList",
        "width"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'translate'
        QtMocHelpers::MethodData<void(double, double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 },
        }}),
        // Method 'translated'
        QtMocHelpers::MethodData<QGeoPath(double, double) const>(5, 2, QMC::AccessPublic, 0x80000000 | 0, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 },
        }}),
        // Method 'length'
        QtMocHelpers::MethodData<double(qsizetype, qsizetype) const>(6, 2, QMC::AccessPublic, QMetaType::Double, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 7, 9 },
        }}),
        // Method 'length'
        QtMocHelpers::MethodData<double(qsizetype) const>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Double, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'length'
        QtMocHelpers::MethodData<double() const>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Double),
        // Method 'size'
        QtMocHelpers::MethodData<qsizetype() const>(10, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Method 'addCoordinate'
        QtMocHelpers::MethodData<void(const QGeoCoordinate &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'insertCoordinate'
        QtMocHelpers::MethodData<void(qsizetype, const QGeoCoordinate &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 15 }, { 0x80000000 | 12, 13 },
        }}),
        // Method 'replaceCoordinate'
        QtMocHelpers::MethodData<void(qsizetype, const QGeoCoordinate &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 15 }, { 0x80000000 | 12, 13 },
        }}),
        // Method 'coordinateAt'
        QtMocHelpers::MethodData<QGeoCoordinate(qsizetype) const>(17, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { 0x80000000 | 7, 15 },
        }}),
        // Method 'containsCoordinate'
        QtMocHelpers::MethodData<bool(const QGeoCoordinate &) const>(18, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'removeCoordinate'
        QtMocHelpers::MethodData<void(const QGeoCoordinate &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'removeCoordinate'
        QtMocHelpers::MethodData<void(qsizetype)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 15 },
        }}),
        // Method 'toString'
        QtMocHelpers::MethodData<QString() const>(20, 2, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'path'
        QtMocHelpers::PropertyData<QVariantList>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'width'
        QtMocHelpers::PropertyData<qreal>(23, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGeoPath, qt_meta_tag_ZN8QGeoPathE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoPath::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QGeoShape>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QGeoPathE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QGeoPathE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8QGeoPathE_t>.metaTypes,
    nullptr
} };

void QGeoPath::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGeoPath *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->translate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 1: { QGeoPath _r = _t->translated((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoPath*>(_a[0]) = std::move(_r); }  break;
        case 2: { double _r = _t->length((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 3: { double _r = _t->length((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 4: { double _r = _t->length();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 5: { qsizetype _r = _t->size();
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->addCoordinate((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 7: _t->insertCoordinate((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[2]))); break;
        case 8: _t->replaceCoordinate((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[2]))); break;
        case 9: { QGeoCoordinate _r = _t->coordinateAt((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->containsCoordinate((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->removeCoordinate((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 12: _t->removeCoordinate((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 13: { QString _r = _t->toString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->variantPath(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->width(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setVariantPath(*reinterpret_cast<QVariantList*>(_v)); break;
        case 1: _t->setWidth(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
