/****************************************************************************
** Meta object code from reading C++ file 'qgeocircle.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioning/qgeocircle.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeocircle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QGeoCircleE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoCircle::qt_create_metaobjectdata<qt_meta_tag_ZN10QGeoCircleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoCircle",
        "translate",
        "",
        "degreesLatitude",
        "degreesLongitude",
        "translated",
        "extendCircle",
        "QGeoCoordinate",
        "coordinate",
        "toString",
        "center",
        "radius"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'translate'
        QtMocHelpers::MethodData<void(double, double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 },
        }}),
        // Method 'translated'
        QtMocHelpers::MethodData<QGeoCircle(double, double) const>(5, 2, QMC::AccessPublic, 0x80000000 | 0, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 },
        }}),
        // Method 'extendCircle'
        QtMocHelpers::MethodData<void(const QGeoCoordinate &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'toString'
        QtMocHelpers::MethodData<QString() const>(9, 2, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'center'
        QtMocHelpers::PropertyData<QGeoCoordinate>(10, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'radius'
        QtMocHelpers::PropertyData<qreal>(11, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGeoCircle, qt_meta_tag_ZN10QGeoCircleE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoCircle::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QGeoShape>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QGeoCircleE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QGeoCircleE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QGeoCircleE_t>.metaTypes,
    nullptr
} };

void QGeoCircle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGeoCircle *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->translate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 1: { QGeoCircle _r = _t->translated((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoCircle*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->extendCircle((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 3: { QString _r = _t->toString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoCoordinate >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->center(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->radius(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCenter(*reinterpret_cast<QGeoCoordinate*>(_v)); break;
        case 1: _t->setRadius(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
