/****************************************************************************
** Meta object code from reading C++ file 'qgeocoordinate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioning/qgeocoordinate.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeocoordinate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14QGeoCoordinateE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoCoordinate::qt_create_metaobjectdata<qt_meta_tag_ZN14QGeoCoordinateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoCoordinate",
        "distanceTo",
        "",
        "other",
        "azimuthTo",
        "atDistanceAndAzimuth",
        "distance",
        "azimuth",
        "distanceUp",
        "toString",
        "CoordinateFormat",
        "format",
        "latitude",
        "longitude",
        "altitude",
        "isValid",
        "Degrees",
        "DegreesWithHemisphere",
        "DegreesMinutes",
        "DegreesMinutesWithHemisphere",
        "DegreesMinutesSeconds",
        "DegreesMinutesSecondsWithHemisphere"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'distanceTo'
        QtMocHelpers::MethodData<qreal(const QGeoCoordinate &) const>(1, 2, QMC::AccessPublic, QMetaType::QReal, {{
            { 0x80000000 | 0, 3 },
        }}),
        // Method 'azimuthTo'
        QtMocHelpers::MethodData<qreal(const QGeoCoordinate &) const>(4, 2, QMC::AccessPublic, QMetaType::QReal, {{
            { 0x80000000 | 0, 3 },
        }}),
        // Method 'atDistanceAndAzimuth'
        QtMocHelpers::MethodData<QGeoCoordinate(qreal, qreal, qreal) const>(5, 2, QMC::AccessPublic, 0x80000000 | 0, {{
            { QMetaType::QReal, 6 }, { QMetaType::QReal, 7 }, { QMetaType::QReal, 8 },
        }}),
        // Method 'atDistanceAndAzimuth'
        QtMocHelpers::MethodData<QGeoCoordinate(qreal, qreal) const>(5, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 0, {{
            { QMetaType::QReal, 6 }, { QMetaType::QReal, 7 },
        }}),
        // Method 'toString'
        QtMocHelpers::MethodData<QString(enum CoordinateFormat) const>(9, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Method 'toString'
        QtMocHelpers::MethodData<QString() const>(9, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'latitude'
        QtMocHelpers::PropertyData<double>(12, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'longitude'
        QtMocHelpers::PropertyData<double>(13, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'altitude'
        QtMocHelpers::PropertyData<double>(14, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'isValid'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CoordinateFormat'
        QtMocHelpers::EnumData<enum CoordinateFormat>(10, 10, QMC::EnumFlags{}).add({
            {   16, CoordinateFormat::Degrees },
            {   17, CoordinateFormat::DegreesWithHemisphere },
            {   18, CoordinateFormat::DegreesMinutes },
            {   19, CoordinateFormat::DegreesMinutesWithHemisphere },
            {   20, CoordinateFormat::DegreesMinutesSeconds },
            {   21, CoordinateFormat::DegreesMinutesSecondsWithHemisphere },
        }),
    };
    return QtMocHelpers::metaObjectData<QGeoCoordinate, qt_meta_tag_ZN14QGeoCoordinateE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoCoordinate::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QGeoCoordinateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QGeoCoordinateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14QGeoCoordinateE_t>.metaTypes,
    nullptr
} };

void QGeoCoordinate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGeoCoordinate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { qreal _r = _t->distanceTo((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        case 1: { qreal _r = _t->azimuthTo((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        case 2: { QGeoCoordinate _r = _t->atDistanceAndAzimuth((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 3: { QGeoCoordinate _r = _t->atDistanceAndAzimuth((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->toString((*reinterpret_cast<std::add_pointer_t<enum CoordinateFormat>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->toString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->latitude(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->longitude(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->altitude(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLatitude(*reinterpret_cast<double*>(_v)); break;
        case 1: _t->setLongitude(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setAltitude(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
