/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativeposition_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioningquick/qdeclarativeposition_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativeposition_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20QDeclarativePositionE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativePosition::qt_create_metaobjectdata<qt_meta_tag_ZN20QDeclarativePositionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativePosition",
        "QML.Element",
        "Position",
        "QML.AddedInVersion",
        "1280",
        "latitudeValid",
        "longitudeValid",
        "altitudeValid",
        "coordinate",
        "QGeoCoordinate",
        "timestamp",
        "speed",
        "speedValid",
        "horizontalAccuracy",
        "verticalAccuracy",
        "horizontalAccuracyValid",
        "verticalAccuracyValid",
        "directionValid",
        "direction",
        "verticalSpeedValid",
        "verticalSpeed",
        "magneticVariation",
        "magneticVariationValid",
        "directionAccuracy",
        "directionAccuracyValid"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'latitudeValid'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'longitudeValid'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'altitudeValid'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'coordinate'
        QtMocHelpers::PropertyData<QGeoCoordinate>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Bindable),
        // property 'timestamp'
        QtMocHelpers::PropertyData<QDateTime>(10, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'speed'
        QtMocHelpers::PropertyData<double>(11, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'speedValid'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'horizontalAccuracy'
        QtMocHelpers::PropertyData<qreal>(13, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'verticalAccuracy'
        QtMocHelpers::PropertyData<qreal>(14, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'horizontalAccuracyValid'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'verticalAccuracyValid'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'directionValid'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable, -1, 0x501),
        // property 'direction'
        QtMocHelpers::PropertyData<double>(18, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Bindable, -1, 0x501),
        // property 'verticalSpeedValid'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable, -1, 0x501),
        // property 'verticalSpeed'
        QtMocHelpers::PropertyData<double>(20, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Bindable, -1, 0x501),
        // property 'magneticVariation'
        QtMocHelpers::PropertyData<double>(21, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Bindable, -1, 0x502),
        // property 'magneticVariationValid'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable, -1, 0x502),
        // property 'directionAccuracy'
        QtMocHelpers::PropertyData<double>(23, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Bindable, -1, 0x603),
        // property 'directionAccuracyValid'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable, -1, 0x603),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativePosition, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativePosition::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QDeclarativePositionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QDeclarativePositionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QDeclarativePositionE_t>.metaTypes,
    nullptr
} };

void QDeclarativePosition::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativePosition *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoCoordinate >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isLatitudeValid(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isLongitudeValid(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isAltitudeValid(); break;
        case 3: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->coordinate(); break;
        case 4: *reinterpret_cast<QDateTime*>(_v) = _t->timestamp(); break;
        case 5: *reinterpret_cast<double*>(_v) = _t->speed(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isSpeedValid(); break;
        case 7: *reinterpret_cast<qreal*>(_v) = _t->horizontalAccuracy(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->verticalAccuracy(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isHorizontalAccuracyValid(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isVerticalAccuracyValid(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isDirectionValid(); break;
        case 12: *reinterpret_cast<double*>(_v) = _t->direction(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isVerticalSpeedValid(); break;
        case 14: *reinterpret_cast<double*>(_v) = _t->verticalSpeed(); break;
        case 15: *reinterpret_cast<double*>(_v) = _t->magneticVariation(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isMagneticVariationValid(); break;
        case 17: *reinterpret_cast<double*>(_v) = _t->directionAccuracy(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isDirectionAccuracyValid(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 0: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableLatitudeValid(); break;
        case 1: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableLongitudeValid(); break;
        case 2: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableAltitudeValid(); break;
        case 3: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableCoordinate(); break;
        case 4: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableTimestamp(); break;
        case 5: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableSpeed(); break;
        case 6: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableSpeedValid(); break;
        case 7: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableHorizontalAccuracy(); break;
        case 8: *static_cast<QUntypedBindable *>(_a[0]) = _t->binableVerticalAccuracy(); break;
        case 9: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableHorizontalAccuracyValid(); break;
        case 10: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableVerticalAccuracyValid(); break;
        case 11: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableDirectionValid(); break;
        case 12: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableDirection(); break;
        case 13: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableVerticalSpeedValid(); break;
        case 14: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableVerticalSpeed(); break;
        case 15: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableMagneticVariation(); break;
        case 16: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableMagneticVariationValid(); break;
        case 17: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableDirectionAccuracy(); break;
        case 18: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableDirectionAccuracyValid(); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativePosition::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativePosition::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QDeclarativePositionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativePosition::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
