/****************************************************************************
** Meta object code from reading C++ file 'qgeopositioninfosource.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioning/qgeopositioninfosource.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeopositioninfosource.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22QGeoPositionInfoSourceE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoPositionInfoSource::qt_create_metaobjectdata<qt_meta_tag_ZN22QGeoPositionInfoSourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoPositionInfoSource",
        "positionUpdated",
        "",
        "QGeoPositionInfo",
        "update",
        "errorOccurred",
        "QGeoPositionInfoSource::Error",
        "supportedPositioningMethodsChanged",
        "startUpdates",
        "stopUpdates",
        "requestUpdate",
        "timeout",
        "updateInterval",
        "minimumUpdateInterval",
        "sourceName",
        "preferredPositioningMethods",
        "PositioningMethods",
        "Error",
        "AccessError",
        "ClosedError",
        "UnknownSourceError",
        "NoError",
        "UpdateTimeoutError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'positionUpdated'
        QtMocHelpers::SignalData<void(const QGeoPositionInfo &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QGeoPositionInfoSource::Error)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'supportedPositioningMethodsChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startUpdates'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stopUpdates'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestUpdate'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'requestUpdate'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'updateInterval'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Bindable),
        // property 'minimumUpdateInterval'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'sourceName'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'preferredPositioningMethods'
        QtMocHelpers::PropertyData<PositioningMethods>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(17, 17, QMC::EnumFlags{}).add({
            {   18, Error::AccessError },
            {   19, Error::ClosedError },
            {   20, Error::UnknownSourceError },
            {   21, Error::NoError },
            {   22, Error::UpdateTimeoutError },
        }),
    };
    return QtMocHelpers::metaObjectData<QGeoPositionInfoSource, qt_meta_tag_ZN22QGeoPositionInfoSourceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoPositionInfoSource::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QGeoPositionInfoSourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QGeoPositionInfoSourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22QGeoPositionInfoSourceE_t>.metaTypes,
    nullptr
} };

void QGeoPositionInfoSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGeoPositionInfoSource *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->positionUpdated((*reinterpret_cast<std::add_pointer_t<QGeoPositionInfo>>(_a[1]))); break;
        case 1: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QGeoPositionInfoSource::Error>>(_a[1]))); break;
        case 2: _t->supportedPositioningMethodsChanged(); break;
        case 3: _t->startUpdates(); break;
        case 4: _t->stopUpdates(); break;
        case 5: _t->requestUpdate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->requestUpdate(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoPositionInfo >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QGeoPositionInfoSource::*)(const QGeoPositionInfo & )>(_a, &QGeoPositionInfoSource::positionUpdated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoPositionInfoSource::*)(QGeoPositionInfoSource::Error )>(_a, &QGeoPositionInfoSource::errorOccurred, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoPositionInfoSource::*)()>(_a, &QGeoPositionInfoSource::supportedPositioningMethodsChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->updateInterval(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->minimumUpdateInterval(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->sourceName(); break;
        case 3: *reinterpret_cast<PositioningMethods*>(_v) = _t->preferredPositioningMethods(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUpdateInterval(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setPreferredPositioningMethods(*reinterpret_cast<PositioningMethods*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 0: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableUpdateInterval(); break;
        case 3: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindablePreferredPositioningMethods(); break;
        default: break;
        }
    }
}

const QMetaObject *QGeoPositionInfoSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGeoPositionInfoSource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QGeoPositionInfoSourceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QGeoPositionInfoSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QGeoPositionInfoSource::positionUpdated(const QGeoPositionInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QGeoPositionInfoSource::errorOccurred(QGeoPositionInfoSource::Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QGeoPositionInfoSource::supportedPositioningMethodsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
