/****************************************************************************
** Meta object code from reading C++ file 'qgeoshape.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioning/qgeoshape.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeoshape.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9QGeoShapeE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoShape::qt_create_metaobjectdata<qt_meta_tag_ZN9QGeoShapeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoShape",
        "contains",
        "",
        "QGeoCoordinate",
        "coordinate",
        "boundingGeoRectangle",
        "QGeoRectangle",
        "toString",
        "other",
        "type",
        "ShapeType",
        "isValid",
        "isEmpty",
        "center",
        "UnknownType",
        "RectangleType",
        "CircleType",
        "PathType",
        "PolygonType"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'contains'
        QtMocHelpers::MethodData<bool(const QGeoCoordinate &) const>(1, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'boundingGeoRectangle'
        QtMocHelpers::MethodData<QGeoRectangle() const>(5, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Method 'toString'
        QtMocHelpers::MethodData<QString() const>(7, 2, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<enum ShapeType>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'isValid'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isEmpty'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'center'
        QtMocHelpers::PropertyData<QGeoCoordinate>(13, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ShapeType'
        QtMocHelpers::EnumData<enum ShapeType>(10, 10, QMC::EnumFlags{}).add({
            {   14, ShapeType::UnknownType },
            {   15, ShapeType::RectangleType },
            {   16, ShapeType::CircleType },
            {   17, ShapeType::PathType },
            {   18, ShapeType::PolygonType },
        }),
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor(const QGeoShape &)>(2, QMC::AccessPublic, {{
            { 0x80000000 | 0, 8 },
        }}),
    };
    return QtMocHelpers::metaObjectData<QGeoShape, qt_meta_tag_ZN9QGeoShapeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject QGeoShape::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QGeoShapeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QGeoShapeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9QGeoShapeE_t>.metaTypes,
    nullptr
} };

void QGeoShape::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGeoShape *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { QGeoShape *_r = new QGeoShape((*reinterpret_cast<std::add_pointer_t<QGeoShape>>(_a[1])));
            if (_a[0]) *reinterpret_cast<void**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) QGeoShape((*reinterpret_cast<std::add_pointer_t<QGeoShape>>(_a[1]))); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->contains((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { QGeoRectangle _r = _t->boundingGeoRectangle();
            if (_a[0]) *reinterpret_cast<QGeoRectangle*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->toString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoCoordinate >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum ShapeType*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isEmpty(); break;
        case 3: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->center(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
