/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativepositionsource_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtpositioning/src/positioningquick/qdeclarativepositionsource_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativepositionsource_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26QDeclarativePositionSourceE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativePositionSource::qt_create_metaobjectdata<qt_meta_tag_ZN26QDeclarativePositionSourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativePositionSource",
        "QML.Element",
        "PositionSource",
        "QML.AddedInVersion",
        "1280",
        "DefaultProperty",
        "parameters",
        "positionChanged",
        "",
        "activeChanged",
        "updateIntervalChanged",
        "supportedPositioningMethodsChanged",
        "preferredPositioningMethodsChanged",
        "sourceErrorChanged",
        "nameChanged",
        "validityChanged",
        "update",
        "timeout",
        "start",
        "stop",
        "positionUpdateReceived",
        "QGeoPositionInfo",
        "sourceErrorReceived",
        "QGeoPositionInfoSource::Error",
        "error",
        "onParameterInitialized",
        "notifySupportedPositioningMethodsChanged",
        "setBackendProperty",
        "name",
        "QVariant",
        "value",
        "backendProperty",
        "position",
        "QDeclarativePosition*",
        "active",
        "valid",
        "updateInterval",
        "supportedPositioningMethods",
        "PositioningMethods",
        "preferredPositioningMethods",
        "sourceError",
        "SourceError",
        "QQmlListProperty<QDeclarativePluginParameter>",
        "PositioningMethod",
        "NoPositioningMethods",
        "SatellitePositioningMethods",
        "NonSatellitePositioningMethods",
        "AllPositioningMethods",
        "AccessError",
        "ClosedError",
        "UnknownSourceError",
        "NoError",
        "UpdateTimeoutError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateIntervalChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'supportedPositioningMethodsChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'preferredPositioningMethodsChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceErrorChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'validityChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void(int)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(16, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'positionUpdateReceived'
        QtMocHelpers::SlotData<void(const QGeoPositionInfo &)>(20, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 16 },
        }}),
        // Slot 'sourceErrorReceived'
        QtMocHelpers::SlotData<void(const QGeoPositionInfoSource::Error)>(22, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'onParameterInitialized'
        QtMocHelpers::SlotData<void()>(25, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'notifySupportedPositioningMethodsChanged'
        QtMocHelpers::SlotData<void()>(26, 8, QMC::AccessPrivate, QMetaType::Void),
        // Method 'setBackendProperty'
        QtMocHelpers::RevisionedMethodData<bool(const QString &, const QVariant &)>(27, 8, QMC::AccessPublic, 0x50e, QMetaType::Bool, {{
            { QMetaType::QString, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'backendProperty'
        QtMocHelpers::RevisionedMethodData<QVariant(const QString &) const>(31, 8, QMC::AccessPublic, 0x50e, 0x80000000 | 29, {{
            { QMetaType::QString, 28 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'position'
        QtMocHelpers::PropertyData<QDeclarativePosition*>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Bindable, 0),
        // property 'active'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Bindable, 1),
        // property 'valid'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable, 7),
        // property 'updateInterval'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'supportedPositioningMethods'
        QtMocHelpers::PropertyData<PositioningMethods>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Bindable, 3),
        // property 'preferredPositioningMethods'
        QtMocHelpers::PropertyData<PositioningMethods>(39, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'sourceError'
        QtMocHelpers::PropertyData<enum SourceError>(40, 0x80000000 | 41, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Bindable, 5),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Bindable, 6),
        // property 'parameters'
        QtMocHelpers::PropertyData<QQmlListProperty<QDeclarativePluginParameter>>(6, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, -1, 0x50e),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PositioningMethod'
        QtMocHelpers::EnumData<enum PositioningMethod>(43, 43, QMC::EnumFlags{}).add({
            {   44, PositioningMethod::NoPositioningMethods },
            {   45, PositioningMethod::SatellitePositioningMethods },
            {   46, PositioningMethod::NonSatellitePositioningMethods },
            {   47, PositioningMethod::AllPositioningMethods },
        }),
        // flag 'PositioningMethods'
        QtMocHelpers::EnumData<PositioningMethods>(38, 43, QMC::EnumIsFlag).add({
            {   44, PositioningMethod::NoPositioningMethods },
            {   45, PositioningMethod::SatellitePositioningMethods },
            {   46, PositioningMethod::NonSatellitePositioningMethods },
            {   47, PositioningMethod::AllPositioningMethods },
        }),
        // enum 'SourceError'
        QtMocHelpers::EnumData<enum SourceError>(41, 41, QMC::EnumFlags{}).add({
            {   48, SourceError::AccessError },
            {   49, SourceError::ClosedError },
            {   50, SourceError::UnknownSourceError },
            {   51, SourceError::NoError },
            {   52, SourceError::UpdateTimeoutError },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativePositionSource, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativePositionSource::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QDeclarativePositionSourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QDeclarativePositionSourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QDeclarativePositionSourceE_t>.metaTypes,
    nullptr
} };

void QDeclarativePositionSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativePositionSource *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->positionChanged(); break;
        case 1: _t->activeChanged(); break;
        case 2: _t->updateIntervalChanged(); break;
        case 3: _t->supportedPositioningMethodsChanged(); break;
        case 4: _t->preferredPositioningMethodsChanged(); break;
        case 5: _t->sourceErrorChanged(); break;
        case 6: _t->nameChanged(); break;
        case 7: _t->validityChanged(); break;
        case 8: _t->update((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->update(); break;
        case 10: _t->start(); break;
        case 11: _t->stop(); break;
        case 12: _t->positionUpdateReceived((*reinterpret_cast<std::add_pointer_t<QGeoPositionInfo>>(_a[1]))); break;
        case 13: _t->sourceErrorReceived((*reinterpret_cast<std::add_pointer_t<QGeoPositionInfoSource::Error>>(_a[1]))); break;
        case 14: _t->onParameterInitialized(); break;
        case 15: _t->notifySupportedPositioningMethodsChanged(); break;
        case 16: { bool _r = _t->setBackendProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { QVariant _r = _t->backendProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoPositionInfo >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePositionSource::*)()>(_a, &QDeclarativePositionSource::positionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePositionSource::*)()>(_a, &QDeclarativePositionSource::activeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePositionSource::*)()>(_a, &QDeclarativePositionSource::updateIntervalChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePositionSource::*)()>(_a, &QDeclarativePositionSource::supportedPositioningMethodsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePositionSource::*)()>(_a, &QDeclarativePositionSource::preferredPositioningMethodsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePositionSource::*)()>(_a, &QDeclarativePositionSource::sourceErrorChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePositionSource::*)()>(_a, &QDeclarativePositionSource::nameChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePositionSource::*)()>(_a, &QDeclarativePositionSource::validityChanged, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativePosition* >(); break;
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QDeclarativePluginParameter> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDeclarativePosition**>(_v) = _t->position(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isActive(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->updateInterval(); break;
        case 4: QtMocHelpers::assignFlags<PositioningMethods>(_v, _t->supportedPositioningMethods()); break;
        case 5: QtMocHelpers::assignFlags<PositioningMethods>(_v, _t->preferredPositioningMethods()); break;
        case 6: *reinterpret_cast<enum SourceError*>(_v) = _t->sourceError(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 8: *reinterpret_cast<QQmlListProperty<QDeclarativePluginParameter>*>(_v) = _t->parameters(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setActive(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setUpdateInterval(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setPreferredPositioningMethods(*reinterpret_cast<PositioningMethods*>(_v)); break;
        case 7: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 0: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindablePosition(); break;
        case 1: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableActive(); break;
        case 2: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableIsValid(); break;
        case 4: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableSupportedPositioningMethods(); break;
        case 6: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableSourceError(); break;
        case 7: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableName(); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativePositionSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativePositionSource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QDeclarativePositionSourceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativePositionSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativePositionSource::positionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativePositionSource::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativePositionSource::updateIntervalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativePositionSource::supportedPositioningMethodsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativePositionSource::preferredPositioningMethodsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativePositionSource::sourceErrorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativePositionSource::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDeclarativePositionSource::validityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
